/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __NNFW_KERNEL_ACL_CLUNIQUETENSOR_H__
#define __NNFW_KERNEL_ACL_CLUNIQUETENSOR_H__

#include <arm_compute/runtime/CL/CLTensor.h>

namespace nnfw {
namespace kernel {
namespace acl {

class CLUniqueTensor
{
public:
  CLUniqueTensor(const ::arm_compute::TensorInfo &info)
  {
    _tensor.allocator()->init(info);
  }

public:
  // Both copy and move are not allowed
  CLUniqueTensor(const CLUniqueTensor &) = delete;
  CLUniqueTensor(CLUniqueTensor &&) = delete;

public:
  ~CLUniqueTensor()
  {
    _tensor.allocator()->free();
  }

public:
  void allocate()
  {
    _tensor.allocator()->allocate();
  }

public:
  ::arm_compute::CLTensor &ref(void) { return _tensor; }
  ::arm_compute::CLTensor *ptr(void) { return &_tensor; }

private:
  ::arm_compute::CLTensor _tensor;
};

} // namespace acl
} // namespace kernel
} // namespace nnfw

#endif //__NNFW_KERNEL_ACL_CLUNIQUETENSOR_H__
