/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <gtest/gtest.h>
#include <OperationsUtils.h>
#include <kernel/acl/nnfw_kernel_acl.h>
#include <kernel/acl/Reshape.h>

// TODO: fix include path in CMakeFiles
#include "util.h"

#ifndef ACL_TEST
#error "ACL_TEST should be defined first!"
#endif // ACL_TEST

#ifndef ACL_CORE_FUNC_NAME
#error "ACL_CORE_FUNC_NAME should be defined first!"
#endif // ACL_CORE_FUNC_NAME

using namespace nnfw::kernel::acl;

ACL_TEST(KernelACL_TC, reshape_1) {
  const nnfw::rt::Shape inputShape = {OperandType::FLOAT32, {1,1,9,1}, 1.0, 0};
  float inputData[9] = {1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f};

  const nnfw::rt::Shape outputShape = { OperandType::FLOAT32, {1,3,3,1}, 1.0, 0 };
  float outputData[9] = {0};

  bool bret = ACL_CORE_FUNC_NAME(inputData, inputShape,
      outputData, outputShape);

  EXPECT_EQ(bret, true);

  float expectData[9] = {1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f, 7.0f, 8.0f, 9.0f};
  bret = util::compareData(outputData, expectData, outputShape);
  EXPECT_EQ(bret, true);

}
