/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <gtest/gtest.h>
#include <OperationsUtils.h>
#include <kernel/acl/nnfw_kernel_acl.h>
#include <kernel/acl/Concatenation.h>

// TODO: fix include path in CMakeFiles
#include "../util.h"

using namespace nnfw::kernel::acl;

TEST(KernelACL_TC, neon_concatFloat32_1)
{
  float inputData_1[6] = {
    1, 2, 3, 4, 5, 6      // [ [ [1],[2],[3] ], [ [4],[5],[6] ] ]
  };
  float inputData_2[6] = {
    7, 8, 9, 10, 11, 12   // [ [ [7],[8],[9] ], [ [10],[11],[12] ] ]
  };
  const nnfw::rt::Shape inputShape_1 = { OperandType::FLOAT32, {1,2,3,1}, 1.0, 0 };
  const nnfw::rt::Shape inputShape_2 = { OperandType::FLOAT32, {1,2,3,1}, 1.0, 0 };
  std::vector<const float*> inputDataPtrs;
  std::vector<nnfw::rt::Shape> inputShapes;
  float outputData[12];
  const nnfw::rt::Shape outputShape = { OperandType::FLOAT32, {1,2,3,2}, 1.0, 0 };
  bool bret;

  inputDataPtrs.push_back(inputData_1);
  inputDataPtrs.push_back(inputData_2);
  inputShapes.push_back(inputShape_1);
  inputShapes.push_back(inputShape_2);

  util::initData(outputData, sizeof(outputData) / sizeof(outputData[0]), 0.0);

  bret = neon::concatenationFloat32(inputDataPtrs, inputShapes, 3, 
                                    outputData, outputShape);
  EXPECT_EQ(bret, true);

  float expectNCHW[] = {
    1, 2, 3, 4, 5, 6,
    7, 8, 9, 10, 11, 12
  };
  float expectData[12]; // [ [ [1,7],[2,8],[3,9] ], [ [4,10],[5,11],[6,12] ] ]
  util::NCHW2NHWC(expectNCHW, expectData, outputShape);
  bret = util::compareData(outputData, expectData, outputShape);
  EXPECT_EQ(bret, true);
}
