/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <cassert>

#include "shape.h"

namespace nnfw {
namespace rt {

// TODO remove from this source and use it from runtime
uint32_t getSizeOfDimension(const Shape& shape, uint32_t dimensionIdx) {
    if (dimensionIdx >= shape.dimensions.size()) {
        // TODO, log the error
        return 0;
    }
    return shape.dimensions[dimensionIdx];
}

} // namespace rt
} // namespace nnfw

namespace nnfw {
namespace kernel {
namespace acl {
namespace util {

arm_compute::TensorShape fromVectorNNShape(const nnfw::rt::Shape& shape)
{
  assert(shape.dimensions.size() == 1);

  const uint32_t len = nnfw::rt::getSizeOfDimension(shape, 0);

  return arm_compute::TensorShape(len);
}

arm_compute::TensorShape fromMatrixNNShape(const nnfw::rt::Shape& shape)
{
  assert(shape.dimensions.size() == 2);

  const uint32_t n = nnfw::rt::getSizeOfDimension(shape, 0);
  const uint32_t c = nnfw::rt::getSizeOfDimension(shape, 1);

  return arm_compute::TensorShape(c, n);
}

arm_compute::TensorShape fromNNShape(const nnfw::rt::Shape& shape)
{
  if( shape.dimensions.size() == 1 )
    return fromVectorNNShape(shape);
  else if( shape.dimensions.size() == 2 )
    return fromMatrixNNShape(shape);

  // TODO: need to treat 3D tensors.

  assert(shape.dimensions.size() == 4);

  // NNAPI assumes the following ordering:
  //
  //  dim(0) -> N
  //  dim(1) -> H
  //  dim(2) -> W
  //  dim(3) -> C
  //
  uint32_t c = nnfw::rt::getSizeOfDimension(shape, 3);
  uint32_t h = nnfw::rt::getSizeOfDimension(shape, 1);
  uint32_t w = nnfw::rt::getSizeOfDimension(shape, 2);
  uint32_t n = nnfw::rt::getSizeOfDimension(shape, 0);

  return arm_compute::TensorShape(w, h, c, n);
}

} // namespace util
} // namespace acl
} // namespace kernel
} // namespace nnfw
