/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All Rights Reserved
 * Copyright (C) 2017 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __NNFW_RT_COMPILATION_BUILDER_H__
#define __NNFW_RT_COMPILATION_BUILDER_H__

#include "NeuralNetworks.h"

namespace nnfw {
namespace rt {

class ExecutionBuilder;
class ModelBuilder;

class CompilationBuilder {
public:
    friend class ExecutionBuilder;  // TODO remove this

    CompilationBuilder(const ModelBuilder* model);

    int setPreference(int32_t preference);

    int finish();

    int createExecution(ExecutionBuilder** execution);

private:
    const ModelBuilder* mModel;

    // Whether the application prefers to go fast or use low power for this execution.
    int32_t mPreference = ANEURALNETWORKS_PREFER_FAST_SINGLE_ANSWER;

    // Once the compilation has been finished, we should not allow further
    // modifications to the compilation.
    bool mFinished = false;
};

} // namespace rt
} // namespace nnfw

#endif // __NNFW_RT_COMPILATION_BUILDER_H__
