#include "NeuralNetworks.h"

#include <iostream>

template <typename T>
void printValue(T value[][4])
{
  for(int y = 0; y < 3; y++)
  {
    std::cout << "[";
    for(int x = 0; x < 4; x++)
    {
      std::cout << value[y][x] << " ";
    }
    std::cout << "] ";
  }
  std::cout << std::endl;
}

int main()
{
  // This is a simple test for bring-up stage
  // TODO Remove this file when we have unit tests ready.

  // Create model
  ANeuralNetworksModel* model = NULL;
  ANeuralNetworksModel_create(&model);

  // Example : 3x4 ADD 3x4
  // Define operand type
  ANeuralNetworksOperandType tensor3x4Type;
  tensor3x4Type.type = ANEURALNETWORKS_TENSOR_FLOAT32;
  tensor3x4Type.scale = 0.f;    // These fields are useful for quantized tensors.
  tensor3x4Type.zeroPoint = 0;  // These fields are useful for quantized tensors.
  tensor3x4Type.dimensionCount = 2;
  uint32_t dims[2] = {3, 4};
  tensor3x4Type.dimensions = dims;

  // Define Acivation
  ANeuralNetworksOperandType activationType;
  activationType.type = ANEURALNETWORKS_INT32;
  activationType.scale = 0.f;
  activationType.zeroPoint = 0;
  activationType.dimensionCount = 0;
  activationType.dimensions = NULL;

  // Add operand
  ANeuralNetworksModel_addOperand(model, &tensor3x4Type);  // operand 0
  ANeuralNetworksModel_addOperand(model, &tensor3x4Type);  // operand 1
  ANeuralNetworksModel_addOperand(model, &activationType); // operand 2, activation
  
  ANeuralNetworksModel_addOperand(model, &tensor3x4Type);  // operand 3, output

  // Set operand value
  float opValue[3][4] = {{2,3,4,5},{2,3,4,5},{2,3,4,5}};

  ANeuralNetworksModel_setOperandValue(model, 1, &opValue, sizeof(opValue));
  
  // Set activation
  int32_t nonValue = ANEURALNETWORKS_FUSED_NONE;
  ANeuralNetworksModel_setOperandValue(model, 2, &nonValue, sizeof(nonValue));
  
  // Add operation
  uint32_t addInputIndexes[3] = {1, 0, 2};
  uint32_t addOutputIndexes[1] = {3};
  ANeuralNetworksModel_addOperation(model, ANEURALNETWORKS_ADD, 3, addInputIndexes, 1, addOutputIndexes);

  // Add input/output
  uint32_t modelInputIndexes[1] = {0};
  uint32_t modelOutputIndexes[1] = {3};
  ANeuralNetworksModel_identifyInputsAndOutputs(model, 1, modelInputIndexes, 1, modelOutputIndexes);

  // Finish model
  ANeuralNetworksModel_finish(model);

  // Compile model
  ANeuralNetworksCompilation* compilation;
  ANeuralNetworksCompilation_create(model, &compilation);

  ANeuralNetworksCompilation_setPreference(compilation, ANEURALNETWORKS_PREFER_LOW_POWER);

  ANeuralNetworksCompilation_finish(compilation);

  // Execute model
  ANeuralNetworksExecution* run = NULL;
  ANeuralNetworksExecution_create(compilation, &run);

  float myInput[3][4] = {{1,2,3,4},{1,2,3,4},{1,2,3,4}};
  ANeuralNetworksExecution_setInput(run, 0, NULL, myInput, sizeof(myInput));

  float myOutput[3][4];
  ANeuralNetworksExecution_setOutput(run, 0, NULL, myOutput, sizeof(myOutput));

  // Set event and run
  ANeuralNetworksEvent* run_end = NULL;
  ANeuralNetworksExecution_startCompute(run, &run_end);

  ANeuralNetworksEvent_wait(run_end);
  ANeuralNetworksEvent_free(run_end);
  
  // CleanUp
  ANeuralNetworksExecution_free(run);
  ANeuralNetworksCompilation_free(compilation);
  ANeuralNetworksModel_free(model); 

  // Print
  std::cout << "my input :" << std::endl;
  printValue(myInput);
  
  std::cout << "op value :" << std::endl;
  printValue(opValue);

  std::cout << "my output :" << std::endl;
  printValue(myOutput);
  
  return 0;
}
