#!/bin/bash

# default DOCKER_IMAGE_NAME=nnfw_docker
DOCKER_IMAGE_NAME=${DOCKER_IMAGE_NAME:-nnfw_docker}
echo "Using docker image $DOCKER_IMAGE_NAME"

SCRIPT_ROOT="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

HOST_HOME=$SCRIPT_ROOT/../..
DOCKER_HOME=/home

GIT_SSL_NO_VERIFY=1

DOCKER_VOLUMES+=" -v $HOST_HOME:$DOCKER_HOME"

DOCKER_ENV_VARS+=" -e http_proxy"
DOCKER_ENV_VARS+=" -e no_proxy"
DOCKER_ENV_VARS+=" -e GIT_SSL_NO_VERIFY"

DOCKER_ENV_VARS+=" -e TARGET_ARCH=armv7l"
DOCKER_ENV_VARS+=" -e CROSS_BUILD=1"
DOCKER_ENV_VARS+=" -e ROOTFS_DIR=/opt/rootfs"
DOCKER_ENV_VARS+=" -e TARGET_OS=tizen"

DOCKER_RUN_OPTS="--rm"
DOCKER_RUN_OPTS+=" -w $DOCKER_HOME"

# prepare rootfs
if [[ ! -d $ROOTFS_DIR ]]; then
  echo "cannot find rootfs"
  exit 1
fi

DOCKER_VOLUMES+=" -v $ROOTFS_DIR/:/opt/rootfs"

CMD="make && make install && make build_test_suite"

docker run $DOCKER_RUN_OPTS $DOCKER_ENV_VARS $DOCKER_VOLUMES $DOCKER_IMAGE_NAME sh -c "$CMD"
BUILD_RESULT=$?

# change owner of root dir and delete rootfs dir
NNFW_OWNER_UID=$(stat -c "%u" $HOST_HOME)
NNFW_OWNER_GID=$(stat -c "%g" $HOST_HOME)

CMD="chown -R $NNFW_OWNER_UID:$NNFW_OWNER_GID $DOCKER_HOME"
docker run $DOCKER_RUN_OPTS $DOCKER_ENV_VARS $DOCKER_VOLUMES $DOCKER_IMAGE_NAME sh -c "$CMD"

exit $BUILD_RESULT
