#!/bin/bash

SCRIPT_ROOT="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
DOCKER_IMAGE_NAME=${DOCKER_IMAGE_NAME:-nnfw_docker}

HOST_HOME=$SCRIPT_ROOT/../..
DOCKER_HOME=/home

GIT_SSL_NO_VERIFY=1

DOCKER_VOLUMES+=" -v $HOST_HOME:$DOCKER_HOME"

DOCKER_ENV_VARS+=" -e http_proxy"
DOCKER_ENV_VARS+=" -e no_proxy"
DOCKER_ENV_VARS+=" -e GIT_SSL_NO_VERIFY"

DOCKER_ENV_VARS+=" -e TARGET_ARCH=armv7l"
DOCKER_ENV_VARS+=" -e CROSS_BUILD=1"
DOCKER_ENV_VARS+=" -e ROOTFS_DIR=/opt/rootfs"
DOCKER_ENV_VARS+=" -e EXT_ACL_FOLDER=/opt/libarmcl"

DOCKER_RUN_OPTS="--rm"
DOCKER_RUN_OPTS+=" -w $DOCKER_HOME"

TMP_DIR=$HOST_HOME/tmp

if [ ! -d $SVACE_ANALYZER_DIR ]; then
  echo "cannot find svace-analyzer"
  exit 1
fi

which $SVACE_ANALYZER_DIR/bin/svace
if [[ $? -ne 0 ]]; then
  echo "cannot find svace-analyzer"
  exit 1
fi

pushd $HOST_HOME

# prepare armcl library
if [[ ! -d $ARMCL_DIR ]]; then
  echo "cannot find armcl"
  exit 1
fi

# prepare rootfs
if [[ ! -d $ROOTFS_DIR ]]; then
  echo "cannot find rootfs"
  exit 1
fi

# prepare svace
if [[ ! -f $SVACE_POLICY_FILE ]]; then
  echo "cannot find svace policy"
  exit 1
fi

DOCKER_VOLUMES+=" -v $SVACE_ANALYZER_DIR:/opt/svace-analyzer"
DOCKER_VOLUMES+=" -v $ROOTFS_DIR:/opt/rootfs"
DOCKER_VOLUMES+=" -v $ARMCL_DIR:/opt/libarmcl"

if [ -n "$DOCKER_INTERACTIVE" ]; then
  DOCKER_RUN_OPTS+=" -it"
  CMD="/bin/bash"
else
  CMD="make external_acl tflite && /opt/svace-analyzer/bin/svace init && /opt/svace-analyzer/bin/svace build make runtime testbuild"
fi

docker run $DOCKER_RUN_OPTS $DOCKER_ENV_VARS $DOCKER_VOLUMES $DOCKER_IMAGE_NAME sh -c "$CMD"
BUILD_RESULT=$?

# Newly created files during above docker run can have different ownership.
# This may cause some problems, for example, some jenkins slaves or developers
# can't remove built files due to lack of permission.
# To address this issue, let's change owner of all files
# in NNFW to owner of NNFW.
NNFW_OWNER_UID=$(stat -c "%u" $HOST_HOME)
NNFW_OWNER_GID=$(stat -c "%g" $HOST_HOME)

CMD="chown -R $NNFW_OWNER_UID:$NNFW_OWNER_GID $DOCKER_HOME"
docker run $DOCKER_RUN_OPTS $DOCKER_ENV_VARS $DOCKER_VOLUMES $DOCKER_IMAGE_NAME $CMD

$SVACE_ANALYZER_DIR/bin/svace analyze --warning $SVACE_POLICY_FILE

popd

exit $BUILD_RESULT
