#!/bin/bash

function check_tools() {
    which clang-format-3.9
    if [[ $? -ne 0 ]]; then
        echo "Error: clang-format-3.9 is not available."
        echo "       Please install clang-format-3.9."
        exit 1
    fi

    which yapf
    if [[ $? -ne 0 ]]; then
        echo "Error: yapf is not available."
        echo "       Please install yapf."
        exit 1
    fi
}

function check_cpp_files() {
    DIRECTORIES_TO_BE_TESTED=$1

    # Check c++ files
    CPP_FILES_IN_COMPILER=$(find "${DIRECTORIES_TO_BE_TESTED[@]}" -iname '*.h' -o -iname '*.cpp' -o -iname '*.cc')

    if [[ ${#CPP_FILES_IN_COMPILER} -eq 0 ]]; then
        echo "No cpp files to be checked"
        return
    fi

    CPP_FILES_TO_BE_TESTED=$(git ls-files $CPP_FILES_IN_COMPILER)
    if [[ ${#CPP_FILES_TO_BE_TESTED} -eq 0 ]]; then
        echo "No changed cpp files to be checked"
        return
    fi

    clang-format-3.9 -i $CPP_FILES_TO_BE_TESTED
}

function check_python_files() {
    DIRECTORIES_TO_BE_TESTED=$1

    # Check python files
    PYTHON_FILES_IN_COMPILER=$(find "${DIRECTORIES_TO_BE_TESTED[@]}" -iname '*.py')

    if [[ ${#PYTHON_FILES_IN_COMPILER} -eq 0 ]]; then
        echo "No python files to be checked"
        return
    fi

    PYTHON_FILES_TO_BE_TESTED=$(git ls-files $PYTHON_FILES_IN_COMPILER)
    if [[ ${#PYTHON_FILES_TO_BE_TESTED} -eq 0 ]]; then
        echo "No changed python files to be checked"
        return
    fi

    yapf -i --style='{based_on_style: pep8, column_limit: 90}' $PYTHON_FILES_TO_BE_TESTED
}

echo "Make sure commit all changes before running this checker."

check_tools

DIRECTORIES_TO_BE_TESTED=()

for DIR_TO_BE_TESTED in $(find -name '.FORMATCHECKED' -exec dirname {} \;); do
    DIRECTORIES_TO_BE_TESTED+=("$DIR_TO_BE_TESTED")
done

if [[ ${#DIRECTORIES_TO_BE_TESTED[@]} -eq 0 ]]; then
    echo "No directories to be checked"
    exit 0
fi

check_cpp_files $DIRECTORIES_TO_BE_TESTED
check_python_files $DIRECTORIES_TO_BE_TESTED

git diff > format.patch
PATCHFILE_SIZE=$(stat -c%s format.patch)
if [[ $PATCHFILE_SIZE -ne 0 ]]; then
    echo "[FAILED] Format checker failed and update code to follow convention."
    echo "         You can find changes in format.patch"
    exit 1
else
    echo "[PASSED] Format checker succeed."
    exit 0
fi

echo "Error: Something went wrong."
exit 1
