#!/bin/bash

# This file is based on https://github.sec.samsung.net/STAR/nncc/pull/80

SCRIPT_ROOT="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
NNFW_ROOT=$SCRIPT_ROOT/../..

LCOV_PATH=$(command -v lcov)
GENHTML_PATH=$(command -v genhtml)

if [[ -z "${LCOV_PATH}" ]]; then
  echo "ERROR: 'lcov' is not found"
  exit 255
fi

if [[ -z "${GENHTML_PATH}" ]]; then
  echo "ERROR: 'genhtml' is not found"
  exit 255
fi

OUTPUT_PATH="$1"

if [[ -z "${OUTPUT_PATH}" ]]; then
  OUTPUT_PATH="$NNFW_ROOT/coverage"
fi

if [[ -e "${OUTPUT_PATH}" ]]; then
  echo "ERROR: '${OUTPUT_PATH}' already exists"
  exit 255
fi

mkdir -p "${OUTPUT_PATH}"

RAW_COVERAGE_INFO_PATH="${OUTPUT_PATH}/coverage.raw.info"
LIBS_COVERAGE_INFO_PATH="${OUTPUT_PATH}/coverage.libs.info"
INCLUDE_COVERAGE_INFO_PATH="${OUTPUT_PATH}/coverage.include.info"
RUNTIMES_COVERAGE_INFO_PATH="${OUTPUT_PATH}/coverage.runtimes.info"
TOOLS_COVERAGE_INFO_PATH="${OUTPUT_PATH}/coverage.tools.info"
FINAL_COVERAGE_INFO_PATH="${OUTPUT_PATH}/coverage.info"
HTML_PATH="${OUTPUT_PATH}/html"
COVERTURA_PATH="${OUTPUT_PATH}/nnfw_coverage.xml"

"${LCOV_PATH}" -c -d "${NNFW_ROOT}" -o "${RAW_COVERAGE_INFO_PATH}"
"${LCOV_PATH}" -e "${RAW_COVERAGE_INFO_PATH}" -o "${LIBS_COVERAGE_INFO_PATH}" '/home/libs/*'
"${LCOV_PATH}" -e "${RAW_COVERAGE_INFO_PATH}" -o "${INCLUDE_COVERAGE_INFO_PATH}" '/home/include/*'
"${LCOV_PATH}" -e "${RAW_COVERAGE_INFO_PATH}" -o "${RUNTIMES_COVERAGE_INFO_PATH}" '/home/runtimes/*'
"${LCOV_PATH}" -e "${RAW_COVERAGE_INFO_PATH}" -o "${TOOLS_COVERAGE_INFO_PATH}" '/home/tools/*'
"${LCOV_PATH}" -a "${LIBS_COVERAGE_INFO_PATH}" -a "${INCLUDE_COVERAGE_INFO_PATH}" \
               -a "${RUNTIMES_COVERAGE_INFO_PATH}" -a "${TOOLS_COVERAGE_INFO_PATH}" \
               -o "${FINAL_COVERAGE_INFO_PATH}"
"${LCOV_PATH}" -r "${FINAL_COVERAGE_INFO_PATH}" -o "${FINAL_COVERAGE_INFO_PATH}" '/home/runtimes/tests/*'
"${LCOV_PATH}" -r "${FINAL_COVERAGE_INFO_PATH}" -o "${FINAL_COVERAGE_INFO_PATH}" '/home/runtimes/nn/depend/*'
"${GENHTML_PATH}" "${FINAL_COVERAGE_INFO_PATH}" --output-directory "${HTML_PATH}"

tar -zcf "${OUTPUT_PATH}"/coverage_report.tar.gz "${HTML_PATH}"
$SCRIPT_ROOT/lcov-to-covertura-xml.sh "${FINAL_COVERAGE_INFO_PATH}" -o "${COVERTURA_PATH}"
