#!/usr/bin/env bash
set -x

# Different from other architectures, the result does not contain only RootFS.
#   $__Rootfs/bin     : Toolchains
#   $__Rootfs/sysroot : RootFS

usage()
{
    echo "Usage: $0 [BuildArch] [NDKVersion] [APILevel]"
    echo "BuildArch  : arm or arm64"
    echo "NDKVersion : r16b or higher (Must start with 'r')"
    echo "APILevel   : 27 or higher"
    exit 1
}

__CrossDir=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
__BuildArch=arm64
__NDKVersion=r16b
__APILevel="27"
__UnprocessedBuildArgs=

for i in "$@" ; do
    lowerI="$(echo $i | awk '{print tolower($0)}')"
    case $lowerI in
        -?|-h|--help)
            usage
            exit 1
            ;;
        arm)
            __BuildArch=arm
            ;;
        arm64)
            __BuildArch=arm64
            ;;
        r16b)
            __NDKVersion=r16b
            ;;
        27)
            __APILevel="27"
            ;;
        *)
            __UnprocessedBuildArgs="$__UnprocessedBuildArgs $i"
            ;;
    esac
done

__RootfsDir=${ROOTFS_DIR:-"${__CrossDir}/rootfs/${__BuildArch}.android"}
__TempDir=${TEMP_DIR:-"/tmp"}

NDK_DIR=android-ndk-${__NDKVersion}
NDK_ZIP=${NDK_DIR}-linux-x86_64.zip

echo "Downloading Android NDK"
wget -nv -nc https://dl.google.com/android/repository/$NDK_ZIP -O $__TempDir/$NDK_ZIP

echo "Unzipping Android NDK"
unzip -qq -o $__TempDir/$NDK_ZIP -d $__TempDir

echo "Generating standalone toolchain and rootfs to $__RootfsDir"

$__TempDir/$NDK_DIR/build/tools/make-standalone-toolchain.sh --arch=$__BuildArch --platform=android-$__APILevel --install-dir=$__RootfsDir
